<?php

// This runs NOW (on inclusion) if enabled
if (defined('PROFILING_ENABLED') && PROFILING_ENABLED) {
  list($usec, $sec) = explode(' ', microtime());
  $startTime = ((float)$usec + (float)$sec);
}

// -------------------------------------------------------------------------------
// Valid arguments:
//  seen:  what to mark as seen
//  category:  select category
//  more:  feed to show more of
//  q, contents:  for searching
//
function argsToUrl ($args) {
  $url = '';
  $first = TRUE;
  foreach ($args as $k => $v) {
    $url .= ($first ? '?' : '&amp;') . "$k=" . urlencode($v);
    $first = FALSE;
  }
  return $url;
}

// -------------------------------------------------------------------------------
// Mark all links in an entire category as read.  Joins links and feeds tables.
//
function markCat (&$db, $userid, $catid) {
  $linkTable = DB_PREFIX.$userid.'_links';
  $feedTable = DB_PREFIX.$userid.'_feeds';

  $sql = "UPDATE $linkTable as L, $feedTable as F SET L.state='". feedlink::STATE_SEEN()
    . "' WHERE L.feed_id=F.id AND F.cat_id='$catid' AND L.state='"
    . feedlink::STATE_NEW() ."'";

  return $db->query($sql);
}

// -------------------------------------------------------------------------------
function forceUpdate (&$db, $userid, &$feed) {
  if (MAGPIE_CACHE_ON)
    purge_rss($feed->rss_link);   // clear out cache
  $feed->last_update = '0000-00-00 00:00:00';
  return $feed->update ($db, $userid);
}

// -------------------------------------------------------------------------------
function feedlinkHTML ($feedid, &$link, $newWindow, $prefix = NULL) {
  return '<a href="#" onclick="expand(\''. $feedid .'\',\'L'. $link->id 
    . '\');return false;" title="expand" class="'. feedlink::stateStr($link->state)
    . '">+ </a>'
    . '<a href="redirect.php?id='. $feedid .'&amp;artid='. $link->id
    . '&amp;redirect_link='. urlencode($link->link) .'" class="'
    . feedlink::stateStr($link->state) .'" onclick="marklink(\'L'. $link->id .'\')"'
    . ($newWindow ? ' target="_blank"' : '') .'>'
    . ($prefix ? (htmlspecialchars($prefix).':&nbsp; ') : '') . $link->title
    . '</a><a href="'.$link->link.'"> &nbsp;</a>';
}

// -------------------------------------------------------------------------------
function morelinkHTML ($feedid, $n, $seen) {
  return '<a href="#" onclick="more(\''. $feedid .'\',\''. $n
    .'\');return false" class="'. ($seen ? 'seen' : 'new')
    .'" title="show older articles"><img src="img/more'
    .($seen ? '-seen' : '').'.png" alt="M"/> show more ...</a>';
}

// -------------------------------------------------------------------------------
function viewlinkHTML ($prefs, $args) {
  $s = '<span class="view">view:';

  if (!isset($prefs['V']) || $prefs['V']=='B')
    $s .= '<img src="img/b1-block.png" title="block view" alt="block view" />';
  else
  {
    $args['view'] = 'B';
    $s .= '<a href="'. $_SERVER['PHP_SELF'] . argsToUrl($args)
      .'" title="go to block view"><img src="img/b1-block_nm.png" alt="block view"/></a>';
  }

  if (isset($prefs['V']) && $prefs['V']=='W')
    $s .= '<img src="img/b1-blockw.png" title="wide block view" alt="wide block view" />';
  else
  {
    $args['view'] = 'W';
    $s .= '<a href="'. $_SERVER['PHP_SELF'] . argsToUrl($args)
      .'" title="go to wide block view"><img src="img/b1-blockw_nm.png" alt="wide block view"/></a>';
  }

  if (isset($prefs['V']) && $prefs['V']=='L')
    $s .= '<img src="img/b1-list.png" title="list view" alt="list view" />';
  else
  {
    $args['view'] = 'L';
    $s .= '<a href="'. $_SERVER['PHP_SELF'] . argsToUrl($args)
      .'" title="go to list view"><img src="img/b1-list_nm.png" alt="list view"/></a>';
  }

  return $s . '</span>';
}

// -------------------------------------------------------------------------------
// Sort by: score (default, if enabled), name
//
function sortlinkHTML ($stats, $prefs, $args) {
  if ($stats)
  {
    $s = '<span class="sort">sort:';

    if ($stats && (!isset($prefs['S']) || $prefs['S']=='S'))
    {
      $s .= '<img src="img/b1-sscore.png" title="sorted by score" alt="score" />';
    }
    else
    {
      $args['sort'] = 'S';
      $s .= '<a href="'. $_SERVER['PHP_SELF'] . argsToUrl($args)
        .'" title="sort by score"><img src="img/b1-sscore_nm.png" alt="sort by score"/></a>';
    }

    if (isset($prefs['S']) && $prefs['S']=='N')
    {
      $s .= '<img src="img/b1-sname.png" title="sorted by name" alt="name" />';
    }
    else
    {
      $args['sort'] = 'N';
      $s .= '<a href="'. $_SERVER['PHP_SELF'] . argsToUrl($args)
        .'" title="sort by name"><img src="img/b1-sname_nm.png" alt="sort by name"/></a>';
    }

    return $s . '</span>';
  }
  else
  {
    return '';
  }
}

// -------------------------------------------------------------------------------
function helplink ($key, $text = '&lt;help&gt;') {
  $link = otherUrl("help.html#$key");
  return "<a class=\"help\" href=\"$link\" onclick=\"javascript:popHelp('$link');return false\">$text</a>";
}

// -------------------------------------------------------------------------------
// Return FQ URL to other file in same directory as the current one.
// Note: this breaks completely if you use URLs like:  /foo/a.php/bar
//
function otherUrl ($file, $secure = FALSE) {
  return 'http'. ($secure ? 's' : '') .'://'. strip_tags($_SERVER['SERVER_NAME']) .
    rtrim(dirname($_SERVER['PHP_SELF']), '/\\') . "/$file";
} 

// -------------------------------------------------------------------------------
function defaultUrl ($user = NULL) {
  return otherUrl('index.php' .
    ($user && $user->default_cat ? '?category='.$user->default_cat : ''));
}

// -------------------------------------------------------------------------------
// JS is from bookmarklet.php.  This works for both FF and IE.
function bookmarkletUrl ($cat = NULL) {
  $url = otherUrl('book.php'. ($cat ? "?category=$cat->id" : ''));
  $js = 'javascript:e%3Ddocument.createElement%28%27script%27%29%3Be.setAttribute%28%27language%27%2C%27javascript%27%29%3Be.setAttribute%28%27src%27%2C%27XYZZY%27%29%3Bvoid%28document.body.appendChild%28e%29%29%3B';
  return str_replace('XYZZY',$url,$js);
}
function rssRegisterUrl() {
  return htmlspecialchars("javascript:window.navigator.registerContentHandler('application/vnd.mozilla.maybe.feed', '". otherUrl('add.php?new=1&url=%s') ."', 'Rnews')");
}

// -------------------------------------------------------------------------------
function feedInfoHTML (&$feed, $stats) {
  $checked = date('n/j g:ia', strtotime($feed->last_update));
  $added = (strtotime($feed->last_add) > 0)
    ? date('n/j g:ia', strtotime($feed->last_add)) : 'never';
  $str = "As of $checked. Last new $added.";
  if ($stats)
    $str .= '&nbsp; <a href="prefs.php?viewstats=1" title="'
      . "expands:{$feed->stat_expand} clicks:{$feed->stat_click} "
      . "articles:{$feed->stat_total}\">Score: ". $feed->getScore() .'</a>';
  return $str;
}

// -------------------------------------------------------------------------------
// Return ( |L<id>|<feedlink> ..., how many links, whether there is another row,
//   whether next item is seen )
//
function ajaxLinkStr (&$db, &$user, $feedid, $start, $n, $onlyNew) {
  $linkStr = '';
  $i = 0;
  $moreexist = FALSE;
  $moreseen = TRUE;

  $sql = 'SELECT * FROM '. DB_PREFIX.$user->userid ."_links WHERE feed_id='$feedid' ORDER BY id DESC LIMIT $start,".($n+1);

  if ($result = $db->query($sql)) {
    while ($i < $n && ($myrow = mysql_fetch_assoc($result))) {
      $link = new feedlink($myrow);
      if ($onlyNew && $link->state != feedlink::STATE_NEW())
        break;

      $linkStr .= "|L{$link->id}|".
        str_replace ('|', '&#124;', feedlinkHTML ($feedid, $link, $user->new_window));
      $i++;
    }

    if ($i <= $n && ($myrow = mysql_fetch_assoc($result))) {
      $link = new feedlink($myrow);
      $moreseen = ($link->state != feedlink::STATE_NEW());
      $moreexist = TRUE;
    }
  }

  return array ($linkStr, $i, $moreexist, $moreseen);
}

// Disable caching of this page
function noCache() {
  header("Cache-Control: no-cache, must-revalidate");  // http/1.1
  header("Expires: Mon, 1 Jan 2008 05:00:00 GMT");  //past
}

// -------------------------------------------------------------------------------
function mutime() {
  list($usec, $sec) = explode(" ", microtime());
  return ((float)$sec + (float)$usec);
}


