<?php

// Figure out 
//
function grokDBVersion($db) {
  // 0.72 -> 0.80 added the 'name' field to user_prefs
  $sql = 'SELECT name FROM '.DB_PREFIX.'user_prefs LIMIT 1';
  if (!$db->query($sql))
    return '0.72';

  // 0.80 -> 0.90 added the 'keep_stats' field to user_prefs
  $sql = 'SELECT keep_stats FROM '.DB_PREFIX.'user_prefs LIMIT 1';
  if (!$db->query($sql))
    return '0.8x';

  return RNEWS_VERSION;
}

// Return whether an upgrade is needed
//
function isUpgradeNeeded ($curVer) {
  switch ($curVer) {
    case '0.72':
    case '0.80':
    case '0.81':
    case '0.8x':
      return true;
  }
  return false;
}

// Upgrade the database.  Uses transactions (rollback will only work if the storage
// engine supports them).  Returns array of (success, newVersion, message).
//
function upgradeDatabase ($db, $curVer) {
  $upStr = '';

  if ($curVer == '0.72')
  {
    return array(false, $curVer,
      'Sorry, I cannot upgrade from 0.72.  You must export feeds to OPML and re-import into a new installation.');
  }

  if ($curVer == '0.80' || $curVer == '0.81' || $curVer == '0.8x')
  {
    if (!$db->query('BEGIN'))
      return array(false, $curVer,
        'Failed to begin user_prefs transaction: '.$db->error().'.');

    // Add some flags to user_prefs
    $sql = 'ALTER TABLE '.DB_PREFIX.'user_prefs ADD ('.
      'keep_stats TINYINT(1) DEFAULT \'1\', '.
      'expand_one TINYINT(1) DEFAULT \'0\', '.
      'new_window TINYINT(1) DEFAULT \'0\', '.
      'disabled TINYINT(1) DEFAULT \'0\')';

    if (!$db->query($sql)) {
      $s = $db->error();
      $db->query('ROLLBACK');
      return array(false, $curVer, 'Failed to alter user_prefs: '. $s .'.');
    }

    // Upgrade each users' feeds:  stat_total, stat_expand, stat_click
    foreach (user::all($db) as $user)
    {
      $sql1 = 'ALTER TABLE '.DB_PREFIX.$user->userid.'_feeds ADD last_add DATETIME AFTER last_update';
      $sql2 = 'ALTER TABLE '.DB_PREFIX.$user->userid.'_feeds ADD ('.
        'stat_total INT(10) UNSIGNED DEFAULT \'1\', '.
        'stat_expand INT(10) UNSIGNED DEFAULT \'0\', '.
        'stat_click INT(10) UNSIGNED DEFAULT \'0\')';
      $sql3 = 'ALTER TABLE '.DB_PREFIX.$user->userid.'_feeds DROP KEY name';

      if (!$db->query($sql1) || !$db->query($sql2) || !$db->query($sql3)) {
        $s = $db->error();
        $db->query('ROLLBACK');
        return array(false, $curVer,
          'Failed to alter user '.$user->userid.'\'s user_prefs: '. $s .'.');
      }
    }

    if (!$db->query('COMMIT')) {
      $s = $db->error();
      $db->query('ROLLBACK');
      return array(false, $curVer,
        'Failed to commit user_prefs transaction: '. $s .'.');
    }

    // Splay links table into <userid>_links tables, and twiddle fields
    $numLinks = 0;
    foreach (user::all ($db) as $user)
    {
      if (!feedlink::create ($db, $user->userid))
        return array(false, $curVer,
          "Failed to create new feedlink table for {$user->userid}: ".$db->error());

      $sql = 'SELECT * FROM '. DB_PREFIX ."links WHERE src_id LIKE '{$user->userid}%' ORDER BY id";
      if (!($result = $db->query($sql))) 
        return array(false, $curVer,
          "Failed to read old links table for {$user->userid}: ".$db->error());

      $ul = strlen($user->userid);
      while ($row = mysql_fetch_assoc($result))
      {
        $f = new feedlink();
        $f->feed_id = substr($row['src_id'],$ul);
        $f->link = $row['link'];
        $f->title = $row['title'];
        $f->description = $row['description'];
        switch ($row['state'])
        {
          case 'seen': $f->state = feedlink::STATE_SEEN(); break;
          case 'new': $f->state = feedlink::STATE_NEW(); break;
          case 'visited': $f->state = feedlink::STATE_VISITED(); break;
          default: $f->state = feedlink::STATE_SEEN(); break;
        }
        $f->pubdate = $row['pubdate'];

        $numLinks++;

        if (!$f->insert ($db, $user->userid))
          return array(false, $curVer,
            "Failed to add new feedlink {$numLinks} for {$user->userid}: ".$db->error());

        $f = null;
      }

      mysql_free_result($result);
    }

    if (!$db->query('DROP TABLE '.DB_PREFIX.'links'))
      return array(false, $curVer,
        'Failed to drop old links table: '.$db->error().'.');

    $upStr .= " Upgraded $curVer to 0.90, moved $numLinks links.";
    $curVer = '0.90';
  }

  $upStr .= ' At version '. RNEWS_VERSION .'.';
  $curVer = RNEWS_VERSION;

  return array(true, $curVer, $upStr);
}

